@@ -21,18 +21,17 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
 #include <mhash.h>
+/* work around typo in mhash.h */
+#if MHASH_API_VERSION == 20020524
+#define MHASH_WHIRLPOOL MHSH_WHIRLPOOL
+#endif
 #include "Python.h"
 #include "structmember.h"
 
-static char __authors__[] =
+static char __author__[] =
 "The mhash python module was developed by:\n\
 \n\
     Gustavo Niemeyer <niemeyer@conectiva.com>\n\
-\n\
-The mhash library was developed by:\n\
-\n\
-    Nikos Mavroyanopoulos <nmav@hellug.gr>\n\
-    Sascha Schumann <sascha@schumann.cx>\n\
 ";
 
 typedef struct {
@@ -58,11 +57,15 @@ staticforward PyTypeObject MHASH_Type;
 static void
 MHASH_dealloc(MHASHObject *self)
 {
-	if (self->thread)
-		mhash_end(self->thread);
+	if (self->thread) {
+		void *ret = mhash_end(self->thread);
+		if (ret) free(ret);
+	}
 	self->ob_type->tp_free((PyObject *)self);
 }
 
+/* FIXME: There has to be a better way to populate the hashid_array[]. */
+
 hashid hashid_array[] = {
 	MHASH_CRC32,
 	MHASH_MD5,
@@ -81,7 +84,19 @@ hashid hashid_array[] = {
 	MHASH_MD4,
 #if MHASH_API_VERSION >= 20011020
 	MHASH_SHA256,
-	MHASH_ADLER32
+	MHASH_ADLER32,
+#endif
+#if MHASH_API_VERSION >= 20020524
+	MHASH_SHA224,
+	MHASH_SHA512,
+	MHASH_SHA384,
+	MHASH_WHIRLPOOL,
+	MHASH_RIPEMD128,
+	MHASH_RIPEMD256,
+	MHASH_RIPEMD320,
+	MHASH_SNEFRU128,
+	MHASH_SNEFRU256,
+	MHASH_MD2,
 #endif
 };
 
@@ -201,40 +216,40 @@ static PyMethodDef MHASH_methods[] = {
 	{NULL,		NULL}		/* sentinel */
 };
 
-static char MHASH__doc__[] =
-"This is the base class, offering basic hashing functionality. MHASH is\n\
-implmented as a newstyle class. It means you may subclass it in your\n\
-python programs and extend its functionality. Don't forget to call its\n\
-__init__ method if you do this.\n\
-\n\
-Constructor:\n\
-\n\
-MHASH(algorithm [, string])\n\
-\n\
-The first parameter is one of the MHASH_* constants provided in the mhash\n\
-module. This will select the hashing algorithm you want to use. The second\n\
-is a string which will update the hash state. It's useful when you want a\n\
-oneliner hash, such as the following:\n\
-\n\
-MHASH(MHASH_SHA1, \"My hashed string\").hexdigest()\n\
-\n\
-Methods:\n\
-\n\
-update(string)  - Update the hash state with 'string'.\n\
-digest()        - Retrieve the currect digest.\n\
-hexdigest()     - Retrieve the current digest in hex format.\n\
-copy()          - Create a new MHASH object, copying the current state.\n\
-\n\
-Attributes:\n\
-\n\
-type            - Selected algorithm.\n\
-digest_size     - Digest size of the selected algorithm.
+static char MHASH__doc__[] = \
+"This is the base class, offering basic hashing functionality. MHASH is\n \
+implmented as a newstyle class. It means you may subclass it in your\n \
+python programs and extend its functionality. Don't forget to call its\n \
+__init__ method if you do this.\n \
+\n \
+Constructor:\n \
+\n \
+MHASH(algorithm [, string])\n \
+\n \
+The first parameter is one of the MHASH_* constants provided in the mhash\n \
+module. This will select the hashing algorithm you want to use. The second\n \
+is a string which will update the hash state. It's useful when you want a\n \
+oneliner hash, such as the following:\n \
+\n \
+MHASH(MHASH_SHA1, \"My hashed string\").hexdigest()\n \
+\n \
+Methods:\n \
+\n \
+update(string)  - Update the hash state with 'string'.\n \
+digest()        - Retrieve the currect digest.\n \
+hexdigest()     - Retrieve the current digest in hex format.\n \
+copy()          - Create a new MHASH object, copying the current state.\n \
+\n \
+Attributes:\n \
+\n \
+type            - Selected algorithm.\n \
+digest_size     - Digest size of the selected algorithm.\
 ";
 
 statichere PyTypeObject MHASH_Type = {
 	PyObject_HEAD_INIT(NULL)
 	0,			/*ob_size*/
-	"MHASH",		/*tp_name*/
+	"mhash.MHASH",		/*tp_name*/
 	sizeof(MHASHObject),	/*tp_basicsize*/
 	0,			/*tp_itemsize*/
 	(destructor)MHASH_dealloc, /*tp_dealloc*/
@@ -330,7 +345,7 @@ MHASH(MHASH_MD5, \"My secret key\", \"My hashed string\").hexdigest()\n\
 statichere PyTypeObject HMAC_Type = {
 	PyObject_HEAD_INIT(NULL)
 	0,			/*ob_size*/
-	"HMAC",			/*tp_name*/
+	"mhash.HMAC",		/*tp_name*/
 	sizeof(MHASHObject),	/*tp_basicsize*/
 	0,			/*tp_itemsize*/
 	(destructor)MHASH_dealloc, /*tp_dealloc*/
@@ -635,8 +650,8 @@ initmhash(void)
 	PyDict_SetItemString(d, "MHASH", (PyObject *)&MHASH_Type);
 	Py_INCREF(&HMAC_Type);
 	PyDict_SetItemString(d, "HMAC", (PyObject *)&HMAC_Type);
-	PyDict_SetItemString(d, "__authors__",
-			     PyString_FromString(__authors__));
+	PyDict_SetItemString(d, "__author__",
+			     PyString_FromString(__author__));
 	PyDict_SetItemString(d, "__version__",
 			     PyString_FromString(VERSION));
 
@@ -662,6 +677,15 @@ initmhash(void)
 	INSINT(MHASH_SHA256);
 	INSINT(MHASH_ADLER32);
 #endif
+#if MHASH_API_VERSION >= 20020524
+	INSINT(MHASH_SHA224);
+	INSINT(MHASH_SHA512);
+	INSINT(MHASH_SHA384);
+	INSINT(MHASH_WHIRLPOOL);
+	INSINT(MHASH_RIPEMD128);
+	INSINT(MHASH_RIPEMD256);
+	INSINT(MHASH_RIPEMD320);
+#endif
 
 	/* Keygen algorithms */
 	INSINT(KEYGEN_MCRYPT);
